 # $Header: /u/spert/src/doc/misc/RCS/opcodes.S,v 1.2 1994/11/18 01:33:38 davidj Exp $
 #
 # Torrent assembler supported opcodes
 # davidj - Tue Aug 31 18:06:52 1993

/* Define some register names so this file can actually be assembled */

#define src $0                  /* General destination register */
#define dest $2                 /* General destination register */
#define creg $4                 /* COP control register */
#define ssrc $7                 /* Scalar source register */
#define vsrc $vr1               /* Vector source register */
#define vsrc1 $vr2              /* Vector source register */
#define vsrc2 $vr3              /* Vector source register */
#define vdest $vr4              /* Vector destination register */
#define base $5                 /* Base register for loads/stores */
#define inc $6                  /* Register used for address increments */
#define vindex $vr4             /* Vector index register for load/store */
#define config $8               /* Config. register for fixed point ops */
#define index $9                /* Index register for insert/extract */

 # ALIASES FOR MIPS DEFINED COPROCESSOR INSTRUCTIONS

 # Control register instructions - O_CONTREG

        cfvu    dest, creg              # cfc2 dest, creg
        ctvu    dest, creg              # ctc2 dest, creg


 # COPROCESSOR INSTRUCTIONS DEFINED BY TORRENT ARCHITECTURE

 # Scalar extraction/insertion from/into vector instructions - O_INSEXT_S

        vext.s  dest, vsrc, index       # was VTS
        vins.s  ssrc, vdest, index      # was VFS

 # Vector extraction from vector instructions - O_INSEXT_V

        vext.v  vdest, vsrc, index      # was VTV

 # Vector load/store instructions - incrementing - O_LSI

        lbai.v  vdest, base, inc
        lhai.v  vdest, base, inc
        lwai.v  vdest, base, inc
        lbuai.v vdest, base, inc
        lhuai.v vdest, base, inc
        sbai.v  vdest, base, inc
        shai.v  vdest, base, inc
        swai.v  vdest, base, inc

        lbst.v  vdest, base, inc
        lhst.v  vdest, base, inc
        lwst.v  vdest, base, inc
        lbust.v vdest, base, inc
        lhust.v vdest, base, inc
        sbst.v  vdest, base, inc
        shst.v  vdest, base, inc
        swst.v  vdest, base, inc

 # Vector load/store instructions - indexed - O_LSX

        lbx.v   vdest, base, vindex
        lhx.v   vdest, base, vindex
        lwx.v   vdest, base, vindex
        lbux.v   vdest, base, vindex
        lhux.v   vdest, base, vindex
        sbx.v   vdest, base, vindex
        shx.v   vdest, base, vindex
        swx.v   vdest, base, vindex

 # Vector integer ALU instructions (vv) O_REG_VVV

        sub.vv      vdest, vsrc1, vsrc2
        subu.vv     vdest, vsrc1, vsrc2
        slt.vv      vdest, vsrc1, vsrc2
        sltu.vv     vdest, vsrc1, vsrc2
        seq.vv      vdest, vsrc1, vsrc2
        add.vv      vdest, vsrc1, vsrc2
        addu.vv     vdest, vsrc1, vsrc2
        sllv.vv     vdest, vsrc1, vsrc2
        srlv.vv     vdest, vsrc1, vsrc2
        srav.vv     vdest, vsrc1, vsrc2
        and.vv      vdest, vsrc1, vsrc2
        or.vv       vdest, vsrc1, vsrc2
        xor.vv      vdest, vsrc1, vsrc2
        nor.vv      vdest, vsrc1, vsrc2
        cmveqz.vv   vdest, vsrc1, vsrc2
        cmvltz.vv   vdest, vsrc1, vsrc2
        cmvgtz.vv   vdest, vsrc1, vsrc2
        cmvnez.vv   vdest, vsrc1, vsrc2
        cmvgez.vv   vdest, vsrc1, vsrc2
        cmvlez.vv   vdest, vsrc1, vsrc2

 # Vector integer ALU instructions (vs) - O_REG_VVS

        sub.vs      vdest, vsrc, ssrc
        subu.vs     vdest, vsrc, ssrc
        slt.vs      vdest, vsrc, ssrc
        sltu.vs     vdest, vsrc, ssrc
        seq.vs      vdest, vsrc, ssrc
        add.vs      vdest, vsrc, ssrc
        addu.vs     vdest, vsrc, ssrc
        sllv.vs     vdest, vsrc, ssrc
        srlv.vs     vdest, vsrc, ssrc
        srav.vs     vdest, vsrc, ssrc
        and.vs      vdest, vsrc, ssrc
        or.vs       vdest, vsrc, ssrc
        xor.vs      vdest, vsrc, ssrc    
        nor.vs      vdest, vsrc, ssrc
        cmveqz.vs   vdest, vsrc, ssrc    
        cmvltz.vs   vdest, vsrc, ssrc    
        cmvgtz.vs   vdest, vsrc, ssrc    
        cmvnez.vs   vdest, vsrc, ssrc    
        cmvgez.vs   vdest, vsrc, ssrc    
        cmvlez.vs   vdest, vsrc, ssrc    

 # Vector integer ALU instructions (sv) - O_REG_VSV

        sub.sv      vdest, ssrc, vsrc
        subu.sv     vdest, ssrc, vsrc
        sllv.sv     vdest, ssrc, vsrc
        srlv.sv     vdest, ssrc, vsrc
        srav.sv     vdest, ssrc, vsrc
        slt.sv      vdest, ssrc, vsrc
        sltu.sv     vdest, ssrc, vsrc

 # Flag setting operations (vv) - O_REG_ZVV

        flt.vv      vsrc1, vsrc2        # Really flt.vv $vr0, vsrc1, vsrc2
        fltu.vv     vsrc1, vsrc2        # Really fltu.vv $vr0, vsrc1, vsrc2
        feq.vv      vsrc1, vsrc2        # Really feq.vv $vr0, vsrc1, vsrc2

 # Flag setting operations (vs) - O_REG_ZVS

        flt.vs      vsrc, ssrc          # Really flt.vs $vr0, vsrc, ssrc
        fltu.vs     vsrc, ssrc          # Really fltu.vs $vr0, vsrc, ssrc
        feq.vs      vsrc, ssrc          # Really feq.vs $vr0, vsrc, ssrc

 # Flag setting operations (vs) - O_REG_ZVS

        flt.sv      ssrc, vsrc          # Really flt.sv $vr0, ssrc, vsrc
        fltu.sv     ssrc, vsrc          # Really fltu.sv $vr0, ssrc, vsrc

 # Vector fixed point instructions (vv) - O_FIX_VVVS

        fxadd.vv    vdest, vsrc1, vsrc2, config
        fxsub.vv    vdest, vsrc1, vsrc2, config
        fxmul.vv    vdest, vsrc1, vsrc2, config

 # Vector pipeline instructions (vs) - O_FIX_VVSS

        fxadd.vs    vdest, vsrc, ssrc, config
        fxsub.vs    vdest, vsrc, ssrc, config
        fxmul.vs    vdest, vsrc, ssrc, config

 # Vector pipeline instructions (sv) - O_FIX_VSVS

        fxadd.sv    vdest, ssrc, vsrc, config
        fxsub.sv    vdest, ssrc, vsrc, config
        fxmul.sv    vdest, ssrc, vsrc, config


 # PSEUDO INTRUCTION VARIANTS ON BASIC TORRENT INSTRUCTIONS

 # Unary vector-vector instructions - O_REG_VZV & O_REG_VVZ

        mov.vv      vdest, vsrc         # cmveqz.vv vdest, $vr0, vsrc
        not.vv      vdest, vsrc         # nor.vv vdest, $vr0, vsrc
        neg.vv      vdest, vsrc         # sub.vv vdest, $vr0, vsrc
        negu.vv     vdest, vsrc         # subu.vv vdest, $vr0, vsrc
        seqz.vv     vdest, vsrc         # seq.vv  vdest, $vr0, vsrc
        snez.vv     vdest, vsrc         # sltu.vv vdest, $vr0, vsrc
        sltz.vv     vdest, vsrc         # slt.vv  vdest, vsrc, $vr0
        sgtz.vv     vdest, vsrc         # slt.vv  vdest, $vr0, vsrc

 # Unary vector-scalar instructions - O_REG_VZS & O_REG_VSZ

        mov.vs      vdest, ssrc         # cmveqz.vs vdest, $vr0, ssrc
        not.vs      vdest, ssrc         # nor.vs vdest, $vr0, ssrc
        neg.vs      vdest, ssrc         # sub.vs vdest, $vr0, ssrc
        negu.vs     vdest, ssrc         # subu.vs vdest, $vr0, ssrc
        seqz.vs     vdest, ssrc         # seq.vs  vdest, $vr0, ssrc
        snez.vs     vdest, ssrc         # sltu.vs vdest, $vr0, ssrc
        sltz.vs     vdest, ssrc         # slt.sv  vdest, ssrc, $vr0
        sgtz.vs     vdest, ssrc         # slt.vs  vdest, $vr0, ssrc

 # Unary flag instructions - O_REG_ZZV, O_REG_ZVZ

        feqz.v      vsrc                # feq.vv  $vr0, $vr0, vsrc
        fnez.v      vsrc                # fltu.vv $vr0, $vr0, vsrc
        fltz.v      vsrc                # flt.vv  $vr0, vsrc, $vr0
        fgtz.v      vsrc                # flt.vv  $vr0, $vr0, vsrc

 # Aliases for vector-vector fixed point instructions

        sadd.vv     vdest, vsrc1, vsrc2 # fxadd.vv vdest, vsrc1, vsrc2, $0
        ssub.vv     vdest, vsrc1, vsrc2 # fxsub.vv vdest, vsrc1, vsrc2, $0
        hmul.vv     vdest, vsrc1, vsrc2 # fxmul.vv vdest, vsrc1, vsrc2, $0
        sneg.vv     vdest, vsrc         # fxsub.vv vdest, $vr0, vsrc, $0

 # Aliases for vector-scalar fixed point instructions

        sadd.vs     vdest, vsrc, ssrc   # fxadd.vs vdest, vsrc, ssrc, $0
        ssub.vs     vdest, vsrc, ssrc   # fxsub.vs vdest, vsrc, ssrc, $0
        hmul.vs     vdest, vsrc, ssrc   # fxmul.vs vdest, vsrc, ssrc, $0
        sneg.vs     vdest, ssrc         # fxsub.vs vdest, $vr0, ssrc, $0

 # Aliases for scalar-vector fixed point instructions

        ssub.sv     vdest, ssrc, vsrc   # fxsub.sv vdest, ssrc, vsrc, $0

 # Non-auto-increment vector load/stores

        lb.v        vdest, base         # lbai.v vdest, base, $vr0
        lh.v        vdest, base         # lhai.v vdest, base, $vr0
        lw.v        vdest, base         # lwai.v vdest, base, $vr0
        lbu.v       vdest, base         # lbuai.v vdest, base, $vr0
        lhu.v       vdest, base         # lhuai.v vdest, base, $vr0
        sb.v        vdest, base         # sbai.v vdest, base, $vr0
        sh.v        vdest, base         # shai.v vdest, base, $vr0
        sw.v        vdest, base         # swai.v vdest, base, $vr0
        
 # COPROCESSOR ZERO INSTRUCTIONS

        icinv
        rfe


